#ifndef _IP_NAT_RULE_H
#define _IP_NAT_RULE_H
#include "ip_nat.h"

struct ip_nat_rule;
struct net_device;

#define IP_NAT_BASE_CTL  	80	/* base for NAT socket options */
#define IP_NAT_SO_INSERT	IP_NAT_BASE_CTL
#define IP_NAT_SO_DELETE	(IP_NAT_BASE_CTL+1)
#define IP_NAT_SO_MAX		IP_NAT_SO_DELETE

#define IP_NAT_MAPPING_TYPE_MAX_NAMELEN 16

struct ip_nat_rule_user
{
	struct ip_conntrack_tuple match;
	struct ip_conntrack_tuple mask;

	/* Type of manipulation. */
	enum ip_nat_manip_type maniptype;

	/* Range to map into */
	struct ip_nat_range range;

	/* Nul-terminated string. */
	char typename[IP_NAT_MAPPING_TYPE_MAX_NAMELEN];

	/* Mapping options (if any). */
	unsigned char map_options[0];
};

#ifdef __KERNEL__
struct ip_nat_mapping_type
{
	struct list_head list;

	/* Name of this mapping. */
	const char name[IP_NAT_MAPPING_TYPE_MAX_NAMELEN];

	/* If you want your module count modified as rules use the type,
	   set this. */
	struct module *me;

	/* Will it accept this rule?  Return true or false. */
	int (*check)(const struct ip_nat_rule *rule);

	/* Function to call if we get a match. */
	unsigned int (*setup)(struct ip_conntrack *conntrack,
			      enum ip_nat_manip_type maniptype,
			      struct ip_nat_multi_range *mr,
			      const struct net_device *device,
			      void *moptions);

	/* Size of options. */
	size_t optionlen;
};

struct ip_nat_rule
{
	struct list_head list;

	/* Here's what we match. */
	struct ip_conntrack_tuple match;
	struct ip_conntrack_tuple mask;

	/* Type of manipulation. */
	enum ip_nat_manip_type maniptype;

	/* Our pre-calculated precedence number. */
	u_int32_t precedence;

	/* Pointer to mapping type. */
	struct ip_nat_mapping_type *mtype;
	void *moptions;

	/* Range to map onto */
	struct ip_nat_multi_range mr;
};

extern int ip_nat_register_mapping_type(struct ip_nat_mapping_type *me);
extern void ip_nat_unregister_mapping_type(struct ip_nat_mapping_type *me);

/* Setup info. */
extern unsigned int ip_nat_setup_info(struct ip_conntrack *conntrack,
				      enum ip_nat_manip_type maniptype,
				      struct ip_nat_multi_range *mr,
				      const struct net_device *device);

extern struct ip_nat_rule *find_rule(const struct ip_conntrack_tuple *tuple);

extern int ip_nat_rule_init(void) __init;
extern void ip_nat_rule_cleanup(void);
#endif /* __KERNEL__ */
#endif /* _IP_NAT_RULE_H */
