#ifndef _LOCKHELP_H
#define _LOCKHELP_H
#include <linux/config.h>

#include <linux/spinlock.h>
#include <asm/atomic.h>
#include <linux/smp.h>

/* Header to do help in lock debugging. */

#ifdef DEBUG_CONNTRACK
struct spinlock_debug
{
	spinlock_t l;
	atomic_t locked_by;
};

#define DECLARE_LOCK(l) 						\
struct spinlock_debug l = { SPIN_LOCK_UNLOCKED, ATOMIC_INIT(-1) }
#define DECLARE_LOCK_STATIC(l) 						 \
static struct spinlock_debug l = { SPIN_LOCK_UNLOCKED, ATOMIC_INIT(-1) }
#define DECLARE_LOCK_EXTERN(l) 			\
extern struct spinlock_debug l

#define MUST_BE_LOCKED(l)					\
do { if (atomic_read(&(l)->locked_by) != smp_processor_id())	\
	printk("%s:%u %s unlocked\n", __FILE__, __LINE__, #l);	\
} while(0)

#define MUST_BE_UNLOCKED(l)					\
do { if (atomic_read(&(l)->locked_by) == smp_processor_id())	\
	printk("%s:%u %s locked\n", __FILE__, __LINE__, #l);	\
} while(0)

#define LOCK_IRQSAVE(lk,f)					\
do {								\
	MUST_BE_UNLOCKED(lk);					\
	spin_lock_irqsave(&(lk)->l,f);				\
	atomic_set(&(lk)->locked_by, smp_processor_id());	\
} while(0)

#define UNLOCK_IRQRESTORE(lk,f)			\
do {						\
	MUST_BE_LOCKED(lk);			\
	atomic_set(&(lk)->locked_by, -1);	\
	spin_unlock_irqrestore(&(lk)->l,f);	\
} while(0)
#else
#define DECLARE_LOCK(l) spinlock_t l
#define DECLARE_LOCK_STATIC(l) static spinlock_t l
#define DECLARE_LOCK_EXTERN(l) extern spinlock_t l

#define MUST_BE_LOCKED(l)
#define MUST_BE_UNLOCKED(l)

#define LOCK_IRQSAVE(l,f) spin_lock_irqsave(l,f)
#define UNLOCK_IRQRESTORE(l,f) spin_unlock_irqrestore(l,f)
#endif /*DEBUG_CONNTRACK*/

#endif /* _LOCKHELP_H */
