/* Kernel module to match connection tracking information.
 * GPL (C) 1999  Rusty Russell (rusty@rustcorp.com.au).
 */
#include <linux/module.h>
#include <linux/skbuff.h>
#include "packet-filter/kernel/ip_tables.h"
#include "packet-filter/extentions/ipt_state.h"
EXPORT_NO_SYMBOLS;

/* Map conntrack to state. */
static enum ipt_state statemap[]
= { [IP_CT_NEW + 1] = IPT_STATE_NEW,
    [IP_CT_ESTABLISHED + 1] = IPT_STATE_ESTABLISHED,
    [IP_CT_RELATED + 1] = IPT_STATE_RELATED,
    [IP_CT_ESTABLISHED_SPECIFIES_RELATED + 1] = IPT_STATE_ESTABLISHED
};

static int
match(struct sk_buff *skb,
      const char *indev,
      const char *outdev,
      const union ipt_matchinfo *matchinfo,
      int offset,
      const void *hdr,
      u_int16_t datalen,
      int *hotdrop)
{
	const struct ipt_state_info *sinfo = (void *)matchinfo->other;

	return (statemap[ip_conntrack_findinfo(skb, &h) + 1] == sinfo->state)
		^ invert;
}

static struct ipt_match state_match
= { { NULL, NULL }, "state", &match, NULL, NULL, THIS_MODULE };

int __init init(void)
{
	__MOD_INC_USE_COUNT(ip_conntrack_module);
	return ipt_register_match(&state_match);
}

void __exit cleanup(void)
{
	ipt_unregister_match(&state_match);
	__MOD_DEC_USE_COUNT(ip_conntrack_module);
}

module_init(init);
module_exit(cleanup);
