/* Defragmenting netfilter module.  Whee! */
#define EXPORT_SYMTAB
#include <linux/module.h>
#include <linux/netfilter.h>
#include <linux/netfilter_ipv4.h>
#include <linux/ip.h>
#include <net/ip.h>

static unsigned int ip_defrag_fn(unsigned int hooknum,
				 struct sk_buff **pskb,
				 const struct net_device *in,
				 const struct net_device *out)
{
	(*pskb)->nfcache |= NFC_IP_FRAG;

	if ((*pskb)->nh.iph->frag_off & htons(IP_MF|IP_OFFSET)) {
		*pskb = ip_defrag(*pskb);
		if (!*pskb)
			return NF_STOLEN;
		ip_send_check((*pskb)->nh.iph);
		(*pskb)->nfcache |= NFC_UNKNOWN | NFC_ALTERED;
	}

	return NF_ACCEPT;
}

/* Before filtering on pre-routing and local-out hooks. */
static struct nf_hook_ops ip_defrag_ops[2] 
= { { { NULL, NULL }, ip_defrag_fn, NULL, PF_INET, NF_IP_PRE_ROUTING, -1 },
    { { NULL, NULL }, ip_defrag_fn, NULL, PF_INET, NF_IP_LOCAL_OUT, -1 }
};

int init_module(void)
{
	int ret;

	ret = nf_register_hook(&ip_defrag_ops[0]);
	if (ret == 0) {
		ret = nf_register_hook(&ip_defrag_ops[1]);
		if (ret != 0)
			nf_unregister_hook(&ip_defrag_ops[0]);
	}
	return ret;
}

void cleanup_module(void)
{
	nf_unregister_hook(&ip_defrag_ops[0]);
	nf_unregister_hook(&ip_defrag_ops[1]);
}
