/* Netfilter device read/write tester
 *
 * This [completely useless] program reflects every 3rd icmp echo
 * request (ping) it sees.  Not good for anything, but something
 * visible to test the netfilter device.  Based on a gutted passer.c,
 * in case the code looks familiar.
 *
 * Note that when you test this stuff, you need to have forwarding on,
 * module 'iptables' installed, a iptables forwarding rule that
 * matches (policy ACCEPT for testing), routes that make the traffic
 * go through you netfilter box, ...
 *
 * (C) 1999 Jeff Long <JwL@gis.net>
 */

#include <sys/types.h>
#include <net/if.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>

#include "device/netfilter_dev.h"
#include <linux/netfilter_ipv4.h>

#include <fcntl.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>

#define NETFILTER_DEVICE_NAME 	"/dev/netfilter_ipv4"
#define BUFSIZE 1024

char buf[ BUFSIZE ];

int main(int argc, const char *argv[])
{
  int fd;
  struct nfdev_verdict verdict;
  struct nfdev_head head;
  int count = 0;
  u_int32_t swap;

  struct nfdev_condition wild_cond = { 0xffffffff, 0, 0 };
  struct nfdev_condition pre_routing_cond = { 1<<NF_IP_PRE_ROUTING, 0, 0 };
  
  fd = open(NETFILTER_DEVICE_NAME, O_RDWR);
  if (fd < 0) {
    fprintf(stderr, "opening %s: %s\n", 
	    NETFILTER_DEVICE_NAME, strerror(errno));
    return 1;
  }
  
  /* Starts with "catch-all" condition. */
  if (ioctl(fd, NFDIOSDELCOND, &wild_cond) < 0) {
    perror("deleting wildcard condition");
    return 1;
  }
  
  if (ioctl(fd, NFDIOSADDCOND, &pre_routing_cond) < 0) {
    perror("adding pre-routing condition");
    return 1;
  }
  
  while (read(fd, &head, sizeof(head)) == sizeof(head)) {
    
    if ( (head.pktlen >= sizeof(struct iphdr))
	 &&
	 (read(fd, buf, BUFSIZE) == head.pktlen) ) {
      
      struct iphdr *ip = (struct iphdr *)buf;
      if ( ip->protocol == IPPROTO_ICMP ) {

	struct icmphdr *icmp = (struct icmphdr *)(buf + 4*ip->ihl);
	
	if ( icmp->type == ICMP_ECHO ) {

	  printf( "PING!\n" );
	
	  if ( !(++count % 3) ) {
	    
	    swap = ip->saddr;
	    ip->saddr = ip->daddr;
	    ip->daddr = swap;
	    printf( "  mirrored\n" );
	    
	    lseek( fd, sizeof(head), SEEK_SET );
	    if ( write( fd, buf, head.pktlen ) != head.pktlen )
	      perror( "writeback" );
	  }
	}
      }
    }
    
    verdict = ((struct nfdev_verdict)
	       { NF_ACCEPT, head.pktlen, head.mark, head.reason });
    if (ioctl(fd, NFDIOSVERDICT, &verdict) != 0) {
      fprintf(stderr, "ioctl: %s\n", strerror(errno));
      break;
    }
  }
  return 1;
}
