#! /bin/sh

# Tests for the different parts of ip nat rules.

# Need job control.
set -m

## First, test IP src and dest matching.

../NAT/userspace/ipnatctl -I -s $TAP0NET.2 -d $TAP1NET.2 -b source -t $TAP0NET.4 || exit 1

# Receive four packets.
./rcv_ip 2 4 < /dev/tap1 > $TMPFILE &
sleep 1

# Only one of these should match.
./gen_ip $TAP0NET.2 $TAP1NET.2 100 1 8 0 55 57 > /dev/tap0
./gen_ip $TAP0NET.2 $TAP1NET.3 100 1 8 0 55 57 > /dev/tap0
./gen_ip $TAP0NET.3 $TAP1NET.2 100 1 8 0 55 57 > /dev/tap0
./gen_ip $TAP0NET.3 $TAP1NET.3 100 1 8 0 55 57 > /dev/tap0

../NAT/userspace/ipnatctl -D -s $TAP0NET.2 -d $TAP1NET.2 -b source -t $TAP0NET.4 || exit 1

if wait %./rcv_ip; then :
else
    echo ./rcv_ip failed:
    cat $TMPFILE
    exit 1
fi

for answer in "$TAP0NET.4 $TAP1NET.2 100 1 8 0 55 57" \
	"$TAP0NET.2 $TAP1NET.3 100 1 8 0 55 57" \
	"$TAP0NET.3 $TAP1NET.2 100 1 8 0 55 57" \
	"$TAP0NET.3 $TAP1NET.3 100 1 8 0 55 57" 
do
    read LINE
    if [ "$LINE" != "$answer" ]
    then
	echo Unexpected src/dst answer "$LINE" vs \`"$answer"\'
	exit 1
    fi
done < $TMPFILE

## Now, test protocol matching.
../NAT/userspace/ipnatctl -I -s $TAP0NET.2 -b source -p icmp -t $TAP0NET.4 || exit 1
../NAT/userspace/ipnatctl -I -s $TAP0NET.2 -b source -p udp -t $TAP0NET.5 || exit 1
../NAT/userspace/ipnatctl -I -s $TAP0NET.2 -b source -p tcp -t $TAP0NET.6 || exit 1
../NAT/userspace/ipnatctl -I -s $TAP0NET.2 -b source -p 2 -t $TAP0NET.7 || exit 1

# Receive five packets.
./rcv_ip 2 5 < /dev/tap1 > $TMPFILE &
sleep 1

# These five should go to different mappings
./gen_ip $TAP0NET.2 $TAP1NET.2 100 icmp 8 0 56 57 > /dev/tap0
./gen_ip $TAP0NET.2 $TAP1NET.2 100 udp 1 2 > /dev/tap0
./gen_ip $TAP0NET.2 $TAP1NET.2 100 tcp 1 2 SYN > /dev/tap0
./gen_ip $TAP0NET.2 $TAP1NET.2 100 2 > /dev/tap0
./gen_ip $TAP0NET.2 $TAP1NET.2 100 7 > /dev/tap0

../NAT/userspace/ipnatctl -D -s $TAP0NET.2 -b source -p icmp -t $TAP0NET.4 || exit 1
../NAT/userspace/ipnatctl -D -s $TAP0NET.2 -b source -p udp -t $TAP0NET.5 || exit 1
../NAT/userspace/ipnatctl -D -s $TAP0NET.2 -b source -p tcp -t $TAP0NET.6 || exit 1
../NAT/userspace/ipnatctl -D -s $TAP0NET.2 -b source -p 2 -t $TAP0NET.7 || exit 1

if wait %./rcv_ip; then :
else
    echo ./rcv_ip failed:
    cat $TMPFILE
    exit 1
fi

for answer in "$TAP0NET.4 $TAP1NET.2 100 1 8 0 56 57" \
    "$TAP0NET.5 $TAP1NET.2 100 17 1 2" \
    "$TAP0NET.6 $TAP1NET.2 100 6 1 2 SYN" \
    "$TAP0NET.7 $TAP1NET.2 100 2" \
    "$TAP0NET.2 $TAP1NET.2 100 7"
do
    read LINE
    if [ "$LINE" != "$answer" ]
    then
	echo Unexpected protocol answer \`"$LINE"\' vs \`"$answer"\'
	exit 1
    fi
done < $TMPFILE

exit 0


