/* Simple program to test NAT's SO_ORIGINAL_DST; called after accept(). */
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <errno.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <unistd.h>
#include "NAT/ip_nat.h"

#if !defined(__GLIBC__) || (__GLIBC__ < 2)
typedef unsigned int socklen_t;
#endif

#define IP_PARTS_NATIVE(n)			\
(unsigned int)((n)>>24)&0xFF,			\
(unsigned int)((n)>>16)&0xFF,			\
(unsigned int)((n)>>8)&0xFF,			\
(unsigned int)((n)&0xFF)

#define IP_PARTS(n) IP_PARTS_NATIVE(ntohl(n))

/* Usage: socket, wait-time */
int main(int argc, char *argv[])
{
	int s, conn;
	struct sockaddr_in inaddr = { AF_INET, htons(atoi(argv[1])), { 0 } };
	socklen_t len = sizeof(inaddr);

	alarm(atoi(argv[2]));
	s = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (s < 0) {
		perror("opening socket");
		exit(1);
	}

	if (bind(s, (struct sockaddr *)&inaddr, sizeof(inaddr)) < 0) {
		perror("binding socket");
		exit(1);
	}

	if (listen(s, 1) < 0) {
		perror("listening on socket");
		exit(1);
	}

	conn = accept(s, NULL, 0);
	if (conn < 0) {
		perror("accepting on socket");
		exit(1);
	}

	if (getsockopt(conn, SOL_IP, SO_ORIGINAL_DST, &inaddr, &len) < 0) {
		perror("getsockopt");
		exit(1);
	}

	printf("%u.%u.%u.%u %u\n", 
	       IP_PARTS(inaddr.sin_addr.s_addr), ntohs(inaddr.sin_port));
	exit(0);
}
